from network_hash_gen.base import BaseHash
from crypt import crypt


class Type6(BaseHash):

    salt_length = 8
    salt_chars = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

    @staticmethod
    def hash_salted(password: str, salt: str) -> str:
        return crypt(password, f"$6${salt}")



"""
# network_hash_gen

network_hash_gen provides functions for hashing passwords for network devices
like routers and switches.

Usually there are 3 kinds of functions for each type of hash.

- the `hash` function, which simply takes a password and returns a hash.
- the `hash_seeded` function with takes a hash and a seed for the random
  number generator that generates the salt.
  This always returns the same hash for the same password+seed combination.
  This is useful for generating configs that are diffed at some point to
  prevent changing hashes all the time.
  Usually a combination of the hostname and username is used as a seed.
- the `hash_salted` function which takes a salt directly.
  This function should only be used if you know what you are doing.
  If possible prefer one of the other functions.
  This function does not check whether that salt makes sense/is supported for
  that hash and may have undefined behaviour if a invalid salt is given.

## Example

```
>>> from network_hash_gen.cisco_ios import Type9
>>> Type9.hash("foobar")
'$9$FteIXKc69u9886$JFenYTrYz7kgex.60fbd8kzIg3Y/fE8lhsrtZeiif8k'
>>> Type9.hash_seeded("foobar", "$hostname-$username")
'$9$XpsDCh72ruxTQc$Cm80vIgCAQPhWrLJczX53Z7qVg0AxKui6t8.QbWfBsU'
```
"""



"""
This module provides hash functions for Juniper Junos

Currently only type 1 (md5-based) and type 6 (SHA512-based) are supported.
"""
from .type_1 import Type1
from .type_6 import Type6

__pdoc__ = {}

__all__ = ["Type1", "Type6"]



from typing import Optional
from abc import abstractmethod
from network_hash_gen.utils import _generate_salt as _generate


class BaseHash:
    """
    A base class that implements salt generation and the `hash` and
    `hash_seeded` functions.

    A subclass only needs to specify the `salt_length` and `salt_chars`
    methods and the `hash_salted` function.
    """

    salt_length: int
    """The length of the salt."""

    salt_chars: str
    """A string containing all characters that can be used in a salt."""

    @classmethod
    @abstractmethod
    def hash_salted(cls, password: str, salt: str) -> str:
        """
        calculates a hash from a fiven password and salt.
        If the salt is invalid for the hash, the behaviour is undefined.
        Prefer the `hash` and `hash_seeded` methods over this one if possible.
        """
        pass  # pragma: no cover

    @classmethod
    def _generate_salt(cls, seed: Optional[str] = None) -> str:
        """
        Generates a salt for the hash.

        If `seed` is specified the value is used to initialize the random number
        generator.
        """
        return _generate(cls.salt_chars, cls.salt_length, seed=seed)

    @classmethod
    def hash(cls, password: str) -> str:
        """
        Calculates the hash.

        An appropriate salt is chosen randomly.
        """
        salt = cls._generate_salt()
        return cls.hash_salted(password, salt)

    @classmethod
    def hash_seeded(cls, password: str, seed: str) -> str:
        """
        Calculates a hash with the given seed used for generating a appropriate
        salt.

        Use this function if you have to generate a password multiple times (e.g.
        when generating configs) to generate the same hash every time.
        """
        salt = cls._generate_salt(seed)
        return cls.hash_salted(password, salt)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['network_hash_gen',
 'network_hash_gen.cisco_ios',
 'network_hash_gen.juniper_junos']

package_data = \
{'': ['*']}

install_requires = \
['passlib>=1.7.4,<2.0.0', 'scrypt>=0.8.20,<0.9.0']

setup_kwargs = {
    'name': 'network-hash-gen',
    'version': '0.1.0',
    'description': 'A library to generate hashes for network devices.',
    'long_description': '# Network Hash Gen\n\nGenerating hashes for network devices like routers and switches - with the\noption to specify seeds or salts.\n\n# Currently supported hashes:\n\n- Cisco IOS/IOS-XE\n  - Type 5\n  - Type 9\n- Juniper JunOS\n  - Type 1\n  - Type 6\n\nIf you are missing a hash function, please open an issue.\n\n# Example\n\nThis example generates a hash with a random salt and a hash with a given seed.\nThe first function returns a different hash most of the times while the\nsecond one always returns the same hash value.\n\n``` python3\n>>> from network_hash_gen.cisco_ios import Type9\n>>> Type9.hash("foobar")\n\'$9$FteIXKc69u9886$JFenYTrYz7kgex.60fbd8kzIg3Y/fE8lhsrtZeiif8k\'\n>>> Type9.hash_seeded("foobar", "$hostname-$username")\n\'$9$XpsDCh72ruxTQc$Cm80vIgCAQPhWrLJczX53Z7qVg0AxKui6t8.QbWfBsU\'\n```\n\n# Installation\n\nThis package can be installed from PyPi via pip or whatever you prefer for\ndependency management.\n\n```\npip install network-hash-gen\n```\n\n# Documentation\n\nThe documentation build against the current master branch can be found here:\nhttps://991jo.github.io/network_hash_gen\n\nTo build the documentation for a specific version run\n\n```\npdoc3 --html --template-dir=templates network_hash_gen\n```\n\nThis will generate documentation in a folder called `html`.\n\n# Development\n\n## Setup\n\nThis project uses [poetry](https://python-poetry.org/).\n\nClone this repository, then run\n```\npoetry install\n```\nThis will create a venv and install the dev dependencies.\n\n## Running the tests\n\nThe tests are in the `tests` folder and are executed with\n\n```\npython3 -m unittest\n```\n\nThe unittests are also run via the pre-commit hooks.\n\nTo get test coverage reports [coverage](https://coverage.readthedocs.io/en/latest/)\nis used. Run \n\n```\ncoverage run -m unittest\n```\n\nto run the tests and `coverage report` for a CLI report of `coverage html` to\ngenerate a HTML version of the coverage report.\n\n## Code Formatting\n\nThe code in this repository is formated with [black](https://github.com/psf/black).\nThe default settings are used.\nPlease format the code with black before commiting.\nThis can be done with\n\n```\nblack network_hash_gen/\n```\n\nThe code formatting is also checked (but not executed) in the commit hooks.\n\n## Pre-Commit Hooks\n\nThere are pre-commit hooks that check the code formating and run the unittests.\nTo activate them, link the script in `scripts/pre-commit.sh` into your git hooks\ndirectory:\n\n```\ncd .git/hooks/\nln -s ../../scripts/pre-commit\n```\n',
    'author': 'Johannes Erwerle',
    'author_email': 'jo+network_hash_gen@swagspace.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/991jo/network_hash_gen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
